from django.db import models
from django.contrib.auth.models import User
from  django.template.loader import  render_to_string
# Create your models here
class Link(models.Model):
    STATUS_NORMAL = 1
    STATUS_DELETE = 0
    STATUS_ITEMS = (
        (STATUS_NORMAL,"正常"),
        (STATUS_DELETE,"删除"),
    )
    title = models.CharField(max_length=50,verbose_name="标题")
    created_time = models.DateTimeField(auto_now_add=True, verbose_name="创建时间")
    href = models.URLField(verbose_name="链接")#默认长度为200
    status = models.PositiveIntegerField(default=STATUS_NORMAL, choices=STATUS_ITEMS,verbose_name="状态")
    weight = models.PositiveIntegerField(default=1, choices=zip(range(1,6),range(1,6)),
                                         verbose_name="权重",
                                         help_text="权重高展示顺序靠前",
                                         )
    owner = models.ForeignKey(User,verbose_name="作者",on_delete=models.CASCADE)
    class Meta:
        verbose_name = verbose_name_plural = "友链"


class SideBar(models.Model):
    STATUS_SHOW = 1
    STATUS_HIDE = 0
    STATUS_ITEMS = (
        (STATUS_SHOW, '展示'),
        (STATUS_HIDE, '隐藏'),
    )
    Display_HTML = 1
    Display_LATEST = 2
    Display_HOT = 3
    Display_COMMENT = 4
    SIDE_TYPE = (
        (Display_HTML, 'HTML'),
        (Display_HOT, "最新文章"),
        (Display_LATEST, "最热文章"),
        (Display_COMMENT,"最近评论"),

    )
    title = models.CharField(max_length=50,verbose_name="标题")
    display_type = models.PositiveIntegerField(choices=SIDE_TYPE, default=1, verbose_name="展示类型")
    content = models.CharField(max_length=500, blank=True, verbose_name="内容",
                               help_text="如果设置的不是HTML类型可为空" )
    status = models.PositiveIntegerField(choices=STATUS_ITEMS,default=STATUS_SHOW, verbose_name="状态")
    owner = models.ForeignKey(User, verbose_name="作者",on_delete=models.CASCADE)
    created_time = models.DateTimeField(auto_now_add=True, verbose_name="创建时间")

    class Meta:
        verbose_name = verbose_name_plural = "侧边栏"
    @classmethod
    def get_all(cls):

        return cls.objects.filter(status = cls.STATUS_SHOW)


    @property
    def content_html(self):
        from blog.models import Post
        from comment.models import Comment
        result = ''
        if self.display_type == self.Display_HTML:
            result  = self.content
        elif self.display_type == self.Display_HOT:
            context = {
                'posts': Post.hot_post(with_related= False)
            }
            result = render_to_string('config/block/sidebar_post.html',context)
        elif self.display_type == self.Display_LATEST:
            context = {
                'posts' : Post.latest_posts(with_related= False)
            }
            result = render_to_string('config/block/sidebar_post.html',context)
        elif self.display_type == self.Display_COMMENT:
            context = {
                'comments': Comment.objects.filter(status=Comment.STATUS_NORMAL)
            }
            result = render_to_string('config/block/sidebar_comments.html',context)
        return  result



import uuid
USER_KEY = 'uid'
TEN_YEARS = 60 * 60 * 24 *365 * 10

class UserIDMiddleware:
    def __init__(self,get_response):
        self.get_response = get_response

    def __call__(self,request):
        uid = self.generate_uid(request)
        request.uid = uid
        response = self.get_response(request)
        response.set_cookie(USER_KEY,uid, max_age = TEN_YEARS, httponly = True)
        return response



    def generate_uid(self,request):
        try:
            uid = request.COOKIES[USER_KEY]
        except KeyError:
            uid = uuid.uuid4()
        return uid




from datetime import date
from django.shortcuts import render,get_object_or_404
from .models import Tag,Post,Category
from django.http import  HttpResponse
from  config.models import SideBar
from django.db.models import  Q, F
from django.views.generic import  ListView,DetailView
from django.core.cache import cache
from comment.form import CommentForm
from comment.models import Comment
# Create your views here.
class CommonViewMixin:
    def get_context_data(self,**kwargs):
        context = super().get_context_data(**kwargs)
        context.update({
            'sidebars':SideBar.get_all()
        })
        context.update(Category.get_navs())
        return context

class IndexView(CommonViewMixin,ListView):
    queryset = Post.latest_posts()
    context_object_name = 'post_list'
    paginate_by = 3
    template_name = 'blog/list.html'


class CategoryView(IndexView):
    def get_context_data(self,**kwargs):
        context = super().get_context_data(**kwargs)
        category_id = self.kwargs.get('category_id')
        category = get_object_or_404(Category,pk=category_id)
        context.update({
            'category': category
        })
        return context

    def get_queryset(self):
        queryset = super().get_queryset()
        category_id = self.kwargs.get('category_id')
        return queryset.filter(category_id=category_id)#####为什么呢

class TagView(IndexView):
    def get_context_data(self,**kwargs):
        context = super().get_context_data(**kwargs)
        tag_id = self.kwargs.get('tag_id')
        tag = get_object_or_404(Tag,pk=tag_id)
        context.update({
            'tag': tag
        })
        return  context

    def get_queryset(self):
        queryset = super().get_queryset()
        tags_id = self.kwargs.get('tag_id')
        return queryset.filter(tags__id = tags_id)


class PostDetailView(CommonViewMixin,DetailView):
    queryset = Post.latest_posts()
    template_name = 'blog/detail.html'
    context_object_name = 'post' #或许一般是在父类李构造，用来传递quertset
    pk_url_kwarg = 'post_id'
    def get(self,request,*args, **kwargs):
        response = super().get(request,*args, **kwargs)
        self.handle_visited()
        return response

    def handle_visited(self):
        increase_pv = False
        increase_uv = False
        uid = self.request.uid
        pv_key = 'pv:%s:%s' %(uid,self.request.path)
        uv_key = 'uv:%s:%s:%s' %(uid,str(date.today()),self.request.path)
        if not cache.get(pv_key):
            increase_pv = True
            cache.set(pv_key,1*60)
        if not cache.get(uv_key):
            increase_uv = True
            cache.set(uv_key,24*60*60)
        if increase_pv and increase_uv:
            Post.objects.filter(pk = self.object.id).update( pv = F('pv') + 1, uv = F('uv') + 1)
        elif increase_pv:
            Post.objects.filter(pk = self.object.id).update( pv = F('pv') + 1)
        elif increase_uv:
            Post.objects.filter(pk = self.object.id).update( uv = F('uv') + 1)








class SearchView(IndexView):
    def get_queryset(self):
        queryset = super().get_queryset()
        keyword = self.request.GET.get('keyword')
        if not keyword:
            return queryset
        else:
            return queryset.filter(Q(title__icontains = keyword) | Q(description__icontains = keyword))

    def get_context_data(self,**kwargs):
        context = super().get_context_data(**kwargs)
        keyword = self.request.GET.get('keyword','')
        context.update({
            'keyword': keyword
        })
        return context


class AuthorView(IndexView):
    def get_queryset(self):
        queryset = super().get_queryset()
        author_id = self.kwargs.get('author_id')
        return queryset.filter(owner_id = author_id)






    #获取queryset与传输context给模板的关系
    #self.kwarg.get()的数据为什么是从URL中拿到的


from django.contrib import admin
from .models import Comment
from typeidea.custom_site import custom_site
from typeidea.base_admin import BaseOwnerAdmin
# Register your models here.
@admin.register(Comment,site=custom_site)
class CommentAdmin(admin.ModelAdmin):
    list_display = ('target', 'nickname', 'content', 'website', 'created_time')


# Generated by Django 4.0.2 on 2022-02-11 12:50

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='SideBar',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('title', models.CharField(max_length=50, verbose_name='标题')),
                ('display_type', models.PositiveIntegerField(choices=[(1, 'HTML'), (2, '最新文章'), (3, '最热文章'), (4, '最近评论')], default=1, verbose_name='展示类型')),
                ('content', models.CharField(blank=True, help_text='如果设置的不是HTML类型可为空', max_length=500, verbose_name='内容')),
                ('status', models.PositiveIntegerField(choices=[(1, '展示'), (0, '隐藏')], default=1, verbose_name='状态')),
                ('created_time', models.DateTimeField(auto_now_add=True, verbose_name='创建时间')),
                ('owner', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL, verbose_name='作者')),
            ],
            options={
                'verbose_name': '侧边栏',
                'verbose_name_plural': '侧边栏',
            },
        ),
        migrations.CreateModel(
            name='Link',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('title', models.CharField(max_length=50, verbose_name='标题')),
                ('created_time', models.DateTimeField(auto_now_add=True, verbose_name='创建时间')),
                ('href', models.URLField(verbose_name='链接')),
                ('status', models.PositiveIntegerField(choices=[(1, '正常'), (0, '删除')], default=1, verbose_name='状态')),
                ('weight', models.PositiveIntegerField(choices=[(1, 1), (2, 2), (3, 3), (4, 4), (5, 5)], default=1, help_text='权重高展示顺序靠前', verbose_name='权重')),
                ('owner', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL, verbose_name='作者')),
            ],
            options={
                'verbose_name': '友链',
                'verbose_name_plural': '友链',
            },
        ),
    ]



# Generated by Django 4.0.1 on 2022-02-18 09:11

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('config', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='sidebar',
            name='display_type',
            field=models.PositiveIntegerField(choices=[(1, 'HTML'), (3, '最新文章'), (2, '最热文章'), (4, '最近评论')], default=1, verbose_name='展示类型'),
        ),
    ]



# Generated by Django 4.0.1 on 2022-02-17 03:20

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('blog', '0002_alter_category_is_nav_alter_category_name'),
    ]

    operations = [
        migrations.AddField(
            model_name='post',
            name='pv',
            field=models.PositiveIntegerField(default=1),
        ),
        migrations.AddField(
            model_name='post',
            name='uv',
            field=models.PositiveIntegerField(default=1),
        ),
    ]



from django.shortcuts import render
from django.http import  HttpResponse
from blog.models import Post
from comment.models import Comment
from config.models import SideBar,Link
from blog.views import CommonViewMixin
from django.views.generic import  ListView

# Create your views here.
class LinkListView(CommonViewMixin,ListView):
    queryset = Link.objects.filter(status=Link.STATUS_NORMAL)
    template_name = 'config/links.html'
    context_object_name = 'link_list'





from django import forms
from ckeditor.widgets import CKEditorWidget
from dal import autocomplete
from .models import Tag,Category,Post
class PostAdminForm(forms.ModelForm):
    description = forms.CharField(widget=forms.Textarea, label = '摘要', required= False)
    content = forms.CharField(widget=CKEditorWidget(), label='正文',required=True)
    category = forms.ModelChoiceField(
        queryset= Category.objects.all(),
        widget=autocomplete.ModelSelect2(url='category-autocomplete'),
        label='分类',
    )
    tag = forms.ModelMultipleChoiceField(
        queryset=Tag.objects.all(),
        widget=autocomplete.ModelSelect2Multiple(url='tag-autocomplete'),
        label='标签'
    )
    class Meta:
        model = Post
        fields = ('category', 'tag', 'title', 'description', 'content', 'status')




from django.contrib import  admin

class BaseOwnerAdmin(admin.ModelAdmin):

    # 1.用来自动补充文章等model的owner字段
    # 2，用来针对queryset过滤当前用户的数据
    exclude = ('owner', )

    def get_queryset(self, request):
        qs = super(BaseOwnerAdmin,self).get_queryset(request)
        return qs.filter(owner=request.user)

    def save_model(self, request, obj, form, change):
        obj.owner = request.user
        return super(BaseOwnerAdmin, self).save_model(request, obj, form, change)


from .base import * # NOQA
DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.mysql',
        'NAME': 'typeidea_db',
        'USER' : 'root',
        'PASSWORD': '200307191q',
        'HOST': '127.0.0.1',
        'PORT' : 3306,
        # 'CONN_MAX_AGE': 5 * 60,
        # 'OPTIONS': {'charset': 'utf8mb4'}

    },
}
DEBUG = True

INSTALLED_APPS += [
    'debug_toolbar',

]
MIDDLEWARE += [
    'debug_toolbar.middleware.DebugToolbarMiddleware'
]
INTERNAL_IPS = ['127.0.0.1']





# Generated by Django 4.0.1 on 2022-02-27 08:12

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('comment', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='comment',
            name='target',
            field=models.CharField(max_length=100, verbose_name='评论目标'),
        ),
    ]



from django import template
from comment.form import CommentForm
from comment.models import Comment

register = template.Library()

@register.inclusion_tag('comment/block.html')
def comment_block(target):
    return {
        'target': target,
        'comment_form' : CommentForm,
        'comment_list' : Comment.get_by_target(target)
    }


from django.shortcuts import render,HttpResponseRedirect,redirect
from django.views.generic import  TemplateView
from .form import CommentForm
# Create your views here.
class CommentView(TemplateView):
    http_method_names = ['post']
    template_name = 'comment/result.html'

    def post(self,request,*args, **kwargs):
        comment_form = CommentForm(request.POST)
        target = request.POST.get('target')

        if comment_form.is_valid():
            instance = comment_form.save(commit=False)
            instance.target = target ####不知道为什么不这样就不成了
            instance.save()
            succeed = True

        else:
            succeed = False
        context = {
            'succeed' : succeed,
            'form' : comment_form,
            'target' : target,
        }
        return self.render_to_response(context)








from django.forms import ModelForm,Textarea,EmailInput,URLInput,ValidationError
from django.utils.translation import gettext as _
from .models import Comment
import mistune

class CommentForm(ModelForm):
    class Meta:
        model  = Comment
        fields = ('nickname','email','website','content')
        labels = {
            'nickname': _('昵称'),
            'content': _('内容'),
            'email': _('邮箱'),
            'website' : _('网站'),
        }
        widgets = {
            'nickname' :Textarea(attrs = {'class': 'form-control', 'style': "width: 60%;"}),
            'content' : Textarea(attrs = {'rows' : 6, 'cols' : 60}),
            'email' : EmailInput(attrs = {'class': 'form-control', 'style': "width: 60%;"}),
            'website' : URLInput(attrs = {'class': 'form-control', 'style' : "width: 60%"})
        }
        max_length = {
            'nickname' : 50,
            'email' : 50,
            'website': 100,
            'content': 500,

        }
    def clean_content(self):
        content = self.cleaned_data.get('content')

        if len(content) < 10:
            raise ValidationError('内容太短了')
        content = mistune.markdown(content)
        return content




from django.contrib.syndication.views import Feed
from django.urls import  reverse
from  django.utils.feedgenerator import Rss201rev2Feed
from .models import Post
class ExtendedRSSFeed(Rss201rev2Feed):
    def add_item_elements(self, handler, item):
        super(ExtendedRSSFeed,self).add_item_elements()
        handler.addQuickElement('content:html', item['content_html'])

class LatestPostFeed(Feed):
    feed_type = Rss201rev2Feed
    title = "Typeidea BLog System"
    link = "/rss/"
    description = "typeidea is a blog system power by django"

    def items(self):
        return Post.objects.filter(status=Post.STATUS_NORMAL)[:5]

    def item_title(self,item):
        return item.title

    def item_link(self, item):
        return reverse('post-detail', args=[item.pk])

    def item_description(self, item):
        return item.description

    def item_extra_kwargs(self, item):
        return {'content_html': self.item_content_html(item)}

    def item_content_html(self,item):
        return item.content_html










from .base import  *
DEBUG = False
REDIS_URL = '127.0.0.1:6379:1'
ALLOWED_HOSTS = ['192.168.179.128']

DATABASES = {
    'default': {
        'ENGINE' : 'django.db.backends.mysql',
        'OPTIONS' : {'charset': 'utf8mb4'},
        'NAME' : 'typeidea_db',
        'USER': 'root',
        'PASSWORD': '200307191q',
        'HOST': '<正式数据库IP>',
        'PORT': 3306,
        'CONT_MAX_AGE' : 5 * 60,

    },
}

CACHE = {
    'default':{
        "BACKEND": 'django_redis.cache.RedisCache',
        'LOCATION': REDIS_URL,
        'TIMEOUT' : 300,
        'OPTIONS': {
            #'PASSWORD': '<对应密码>',
            'CLIENT_CLASS': 'django_redis_client.DefaultClient',
            'PARSER_CLASS': 'redis.connection.HiredisParser',
        },
        'CONNECTION_POOL_CLASS': 'redis.connection.BlockingConnectionPool',


    }
}


# Generated by Django 4.0.2 on 2022-02-11 12:50

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('blog', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='Comment',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('content', models.CharField(max_length=2000, verbose_name='内容')),
                ('nickname', models.CharField(max_length=50, verbose_name='昵称')),
                ('website', models.URLField(verbose_name='网站')),
                ('email', models.EmailField(max_length=254, verbose_name='邮箱')),
                ('status', models.PositiveIntegerField(choices=[(1, '正常'), (1, '删除')], default=1, verbose_name='状态')),
                ('created_time', models.DateTimeField(auto_now_add=True, verbose_name='创建时间')),
                ('target', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='blog.post', verbose_name='评论目标')),
            ],
            options={
                'verbose_name': '评论',
                'verbose_name_plural': '评论',
            },
        ),
    ]



from rest_framework import serializers,pagination
from .models import  Post,Category

class PostSerializer(serializers.HyperlinkedModelSerializer):
    url = serializers.HyperlinkedIdentityField(view_name='api-post-detail')

    category = serializers.SlugRelatedField(
        read_only= True,
        slug_field='name',
    )
    tags = serializers.SlugRelatedField(
        many= True,
        read_only=True,
        slug_field='name'
    )
    owner = serializers.SlugRelatedField(
        read_only=True,
        slug_field='username'
    )
    created_time = serializers.DateTimeField(format= "%Y-%m-%d %H:%M:%S")
    class Meta:
        model = Post
        fields = ['url','id', 'title', 'category', 'tags', 'owner', 'created_time']


class PostDetailSerializer(PostSerializer):
    class Meta:
        model = Post
        fields = ['id', 'title', 'category', 'tags', 'owner', 'content_html', 'created_time']


class CategorySerializer(serializers.ModelSerializer):
    class Meta:
        model = Category
        fields = ['id', 'name', 'created_time']


class CategoryDetailSerializer(CategorySerializer):
    posts = serializers.SerializerMethodField('paginated_posts')

    def paginated_posts(self,obj):
        posts = obj.post_set.filter(status = Post.STATUS_NORMAL)
        paginator = pagination.PageNumberPagination()
        page = paginator.paginate_queryset(posts, self.context['request'])
        serializers = PostSerializer(page, many = True, context={'request':
                                                                 self.context['request']})
        return {
            'count': posts.count(),
            'result': serializers.data,
            'previous': paginator.get_previous_link(),
            'next' : paginator.get_next_link(),

        }
    class Meta:
        model = Category
        fields = (
            'id', 'name', 'created_time', 'posts',
        )




# Generated by Django 4.0.2 on 2022-02-11 12:50

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='Category',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=128, verbose_name='名称')),
                ('status', models.IntegerField(choices=[(1, '正常'), (0, '删除')], default=1, verbose_name='状态')),
                ('is_nav', models.BooleanField(default=False, verbose_name='是否为导航')),
                ('created_time', models.DateTimeField(auto_now_add=True, verbose_name='创建时间')),
                ('owner', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL, verbose_name='作者')),
            ],
            options={
                'verbose_name': '分类',
                'verbose_name_plural': '分类',
            },
        ),
        migrations.CreateModel(
            name='Tag',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=10, verbose_name='名称')),
                ('status', models.IntegerField(choices=[(1, '正常'), (0, '删除')], default=1, verbose_name='状态')),
                ('created_time', models.DateTimeField(auto_now_add=True, verbose_name='创建时间')),
                ('owner', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL, verbose_name='作者')),
            ],
            options={
                'verbose_name': '标签',
                'verbose_name_plural': '标签',
            },
        ),
        migrations.CreateModel(
            name='Post',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('title', models.CharField(max_length=255, verbose_name='标题')),
                ('description', models.CharField(blank=True, max_length=1024, verbose_name='摘要')),
                ('content', models.TextField(help_text='正文必须为MARKDOWN格式', verbose_name='正文')),
                ('status', models.IntegerField(choices=[(1, '正常'), (0, '删除'), (2, '草稿')], default=1, verbose_name='状态')),
                ('created_time', models.DateTimeField(auto_now_add=True, verbose_name='创建时间')),
                ('category', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='blog.category', verbose_name='分类')),
                ('owner', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL, verbose_name='作者')),
                ('tags', models.ManyToManyField(to='blog.Tag', verbose_name='标签')),
            ],
            options={
                'verbose_name': '文章',
                'verbose_name_plural': '文章',
                'ordering': ['id'],
            },
        ),
    ]



# Generated by Django 4.0.1 on 2022-02-17 02:30

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('blog', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='category',
            name='is_nav',
            field=models.BooleanField(default=False, verbose_name='是否是导航'),
        ),
        migrations.AlterField(
            model_name='category',
            name='name',
            field=models.CharField(max_length=50, verbose_name='名称'),
        ),
    ]



from django.contrib import admin
from .models import Category,Post,Tag
from django.utils.html import format_html
from django.urls import reverse
from .adminforms import PostAdminForm
from django.contrib import admin
from typeidea.custom_site import custom_site
from typeidea.base_admin import BaseOwnerAdmin
from django.contrib.admin.models import LogEntry

# Register your models here.



class PostInline(admin.TabularInline): #样式不同
    fields = ('title','description')
    extra = 1 #控制额外多几个
    model = Post


@admin.register(Category,site=custom_site)
class CategoryAdmin(BaseOwnerAdmin):
    list_display = ('name','status','is_nav','created_time',)
    fields = ('name', 'status', 'is_nav',)
    inlines = [PostInline,]
    def save_model(self, request, obj, form, change):
        obj.owner = request.user
        return super(CategoryAdmin, self).save_model(request,obj, form, change)


@admin.register(Tag,site=custom_site)
class Tagadmin(BaseOwnerAdmin):
    list_display = ('name', 'status','created_time')
    fields = ('name',"status",)
    def save_model(self, request, obj, form, change):
        obj.owner = request.user
        return super(Tagadmin, self).save_model(request,obj, form, change)


@admin.register(Post,site=custom_site)
class Postadmin(BaseOwnerAdmin):
    class CategoryOwnerFilter(admin.SimpleListFilter):
    #自定义过滤器只展示当前用户分类
        title = "分类过滤器"
        parameter_name = "owner_category"

        def lookups(self, request, model_admin):
            return Category.objects.filter(owner=request.user).values_list('id','name')
        def queryset(self, request, queryset):
            category_id = self.value()
            if category_id:
                return queryset.filter(category_id=category_id)
            return queryset


    list_display = [
        'title', 'category', 'status',
        'created_time', 'operator','description','owner'
    ]
    list_display_links = []
    list_filter = [CategoryOwnerFilter]
    search_fields = ['title', 'category__name']

    actions_on_top = True
    actions_on_bottom = True
    form = PostAdminForm #使描述字段能以某种方式呈现
    #编辑动作
    save_on_top = True
    exclude = ('owner',)
    fieldsets = (
        ('基础配置',{
            'description': '基础配置描述',
            'fields':(
                ('title', 'category'),
                'status' ,
            ),
        }),
        ('内容', {
            'fields': (
                'description',
                'content',
            ),
        }),
        ('额外信息', {
            'classes': ('collapse',),
            'fields': ('tags',)
        })
    )

    def operator(self,obj):
        return format_html(
            '<a href="{}">编辑</a>',
            reverse('cus_admin:blog_post_change', args=(obj.id,))
        )
    operator.short_description = '操作'

    def save_model(self, request, obj, form, change):
        obj.owner = request.user
        return super(Postadmin, self).save_model(request,obj, form, change)
    def get_queryset(self, request):
        qs = super(Postadmin, self).get_queryset(request)
        return qs.filter(owner=request.user)


@admin.register(LogEntry,site=custom_site)
class LogEntryAdmin(admin.ModelAdmin):
    list_display = ['object_repr', 'object_id', 'action_flag', 'user', 'change_message']


from django.contrib.admin import AdminSite

class CustomSide(AdminSite):
    site_header = 'Typeidea'
    site_title = 'Typeidea后台管理系统'
    index_title = "首页"
custom_site = CustomSide(name= 'cus_admin')



from rest_framework import viewsets
from rest_framework.permissions import IsAdminUser

from .models  import  Post,Category
from .serializer import  PostSerializer,PostDetailSerializer,CategorySerializer, CategoryDetailSerializer

class PostViewSet(viewsets.ModelViewSet):
    serializer_class = PostSerializer
    queryset = Post.objects.filter(status = Post.STATUS_NORMAL)
    def retrieve(self, request, *args, **kwargs):
        self.serializer_class = PostDetailSerializer
        return super().retrieve(request, *args, **kwargs)
    # permission_classes = [IsAdminUser]




class CategoryViewSet(viewsets.ModelViewSet):
    serializer_class = CategorySerializer
    queryset =  Category.objects.filter(status = Category.STATUS_NORMAL)

    def retrieve(self, request, *args, **kwargs):
        self.serializer_class = CategoryDetailSerializer
        return super().retrieve(request, *args, **kwargs)





from django.db import models
from blog.models import Post
# Create your models here.

class Comment(models.Model):
    STATUS_NORMAL = 1
    STATUS_DELETE = 0
    STATUS_ITEMS = (
        (STATUS_NORMAL,"正常"),
        (STATUS_NORMAL,'删除'),
    )
    target = models.CharField(max_length=100, verbose_name="评论目标")
    content = models.CharField(max_length=2000,verbose_name="内容",)
    nickname = models.CharField(max_length=50, verbose_name="昵称")
    website = models.URLField(verbose_name="网站")
    email = models.EmailField(verbose_name="邮箱")
    status = models.PositiveIntegerField(choices=STATUS_ITEMS,verbose_name="状态",default=STATUS_NORMAL)
    created_time = models.DateTimeField(auto_now_add=True, verbose_name="创建时间")
    class Meta:
        verbose_name = verbose_name_plural = "评论"

    @classmethod
    def get_by_target(cls,target):
        return cls.objects.filter(target= target, status=cls.STATUS_NORMAL)







from setuptools import setup, find_packages

setup(
    name = 'typeidea_huangst0.2',
    version = '0.1',
    description= 'Blog System base on Django',
    author='huangst',
    author_email='huangst21@lzu.edu.cn',
    url = 'https://www.huangst21.com',
    license='MIT',
    packages= find_packages('typeidea'),
    package_dir={'': 'typeidea'},
    package_data = {'': [
        'themes/*/*/*/*',
    ]},
    extras_require = {
        'ipython': ["ipython==6.2.1"]
    },
    scripts=[
        'typeidea/manage.py'
        ,
    ],
    entry_points = {
        'console_scripts': [
            'typeidea_manage = manage:main',

        ]
    },

    classifiers=[
        'Development Status :: 3 - Alpha',
        'Topic :: Software Development :: Libraries',
        'License :: OSI Approved :: MIT License',

        'Programming Language :: Python :: 3.8',
    ],

)



from django.contrib import admin
from .models import Link,SideBar
from typeidea.custom_site import custom_site
from typeidea.base_admin import BaseOwnerAdmin
# Register your models here.
@admin.register(Link,site=custom_site)
class Linkadmin(BaseOwnerAdmin):
    list_display = ('title', 'href', 'status', 'weight', 'created_time')
    fields = ('title', 'href', 'status', 'weight')
    def save_model(self, request, obj, form, change):
        obj.owner = request.user
        return super(Linkadmin, self).save_model(request,obj, form, change)

@admin.register(SideBar,site=custom_site)
class SideBaradmin(BaseOwnerAdmin):
    list_display = ('title', 'display_type', 'content', 'created_time')
    fields = ('title', 'display_type', 'content')
    def save_model(self, request, obj, form, change):
        obj.owner = request.user
        return super(SideBaradmin, self).save_model(request,obj, form, change)


# Generated by Django 4.0.1 on 2022-03-04 03:28

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('blog', '0003_post_pv_post_uv'),
    ]

    operations = [
        migrations.AddField(
            model_name='post',
            name='content_html',
            field=models.TextField(blank=True, editable=False, verbose_name='正文HTML'),
        ),
    ]



from django.contrib.sitemaps import  Sitemap
from django.urls import reverse

from.models import Post

class PostSiteMap(Sitemap):
    changefreq = "always"
    priority = '1.0'
    protocol = 'https'

    def items(self):
        return Post.objects.filter(status=Post.STATUS_NORMAL)

    def lastmod(self,obj):
        return obj.created_tiem

    def location(self, obj):
        return reverse('post-detail', args=[obj.pk])




"""typeidea URL Configuration

The `urlpatterns` list routes URLs to views. For more information please see:
    https://docs.djangoproject.com/en/4.0/topics/http/urls/
Examples:
Function views
    1. Add an import:  from my_app import views
    2. Add a URL to urlpatterns:  path('', views.home, name='home')
Class-based views
    1. Add an import:  from other_app.views import Home
    2. Add a URL to urlpatterns:  path('', Home.as_view(), name='home')
Including another URLconf
    1. Import the include() function: from django.urls import include, path
    2. Add a URL to urlpatterns:  path('blog/', include('blog.urls'))
"""
from django.contrib import admin
from django.urls import path,re_path,include
from .custom_site import  custom_site
from blog.views import PostDetailView,IndexView,CategoryView,TagView,SearchView,AuthorView
from config.views import  LinkListView
from comment.views import CommentView
from django.contrib.sitemaps import  views as sitemap_views
from blog.rss import  LatestPostFeed
from blog.sitemap import PostSiteMap
from .autocomplete import CategoryAutocomplete,TagAutocomplete
from blog.apis import PostViewSet,CategoryViewSet
from rest_framework .routers import DefaultRouter
from rest_framework.documentation import include_docs_urls
from django.conf import  settings
from django.views.decorators.cache import cache_page

router = DefaultRouter()
router.register(r'post', PostViewSet, basename = 'api-post')
router.register(r'category', CategoryViewSet, basename='api-category')

urlpatterns = [
    path('' ,IndexView.as_view(),name = 'index'),
    re_path(r'^search/$',SearchView.as_view(),name = 'search'),
    re_path(r'^author/(?P<author_id>\d+)/$',AuthorView.as_view(), name = "author"),
    re_path(r'^category/(?P<category_id>\d+)/$',CategoryView.as_view(), name='category-list'),
    re_path(r'^tag/(?P<tag_id>\d+)/$',TagView.as_view(),name = 'tag-list'),
    re_path(r'^post/(?P<post_id>\d+).html$',PostDetailView.as_view(),name = 'post-detail'),
    re_path(r'^links/$',LinkListView.as_view(),name='links'),
    re_path(r'^comment/$',CommentView.as_view(), name = 'comment'),
    re_path(r'^rss|feed/', LatestPostFeed(), name = 'rss'),
    re_path(r'^sitemap\.xml$',cache_page(60*20, key_prefix='sitemap_cache_')(sitemap_views.sitemap),{'sitemaps': {'posts': PostSiteMap}}),
    re_path(r'^category-autocomplete/$',CategoryAutocomplete.as_view(),name = 'category-autocomplete'),
    re_path(r'^tag-autocomplete/$',TagAutocomplete.as_view(), name= 'tag-autocomplete'),
    re_path(r'^api/docs/', include_docs_urls(title='typeidea')),
    path('api/',include(router.urls)),


    path('super_admin/',admin.site.urls,name='super-admin'),
    path('admin/', custom_site.urls,name='admin'),

]


if settings.DEBUG:
    import debug_toolbar
    urlpatterns = [
        re_path(r'^__debug__/', include(debug_toolbar.urls))
    ] + urlpatterns


from django.db import models
from django.contrib.auth.models import User
import mistune
from django.utils.functional import cached_property


class Category(models.Model):
    STATUS_NORMAL = 1
    STATUS_DELETE = 0
    STATUS_ITEMS = (
        (STATUS_NORMAL, "正常"),
        (STATUS_DELETE, "删除"),
    )
    name = models.CharField(max_length=50,verbose_name="名称")
    status = models.IntegerField(choices=STATUS_ITEMS,default=STATUS_NORMAL,verbose_name="状态")
    is_nav = models.BooleanField(default=False, verbose_name="是否是导航")
    owner = models.ForeignKey(User, verbose_name="作者",on_delete=models.CASCADE)
    created_time = models.DateTimeField(auto_now_add=True,verbose_name="创建时间")
    class Meta:
        verbose_name = verbose_name_plural = '分类'
    @classmethod
    def get_navs(cls):
        categories = Category.objects.filter(status=cls.STATUS_NORMAL)
        nav_categories = []
        normal_categories = []
        for cate in categories:
            if cate.is_nav:
                nav_categories.append(cate)
            else:
                normal_categories.append(cate)
        return {
                'navs': nav_categories,
                'categories': normal_categories
            }
    def __str__(self):
        return self.name


class Tag(models.Model):
    STATUS_NORMAL = 1
    STATUS_DELETE = 0
    STATUS_ITEMS = (
        (STATUS_NORMAL, "正常"),
        (STATUS_DELETE, "删除"),
    )
    name = models.CharField(max_length=10,verbose_name="名称")
    status = models.IntegerField(choices=STATUS_ITEMS,default=STATUS_NORMAL,verbose_name="状态")
    created_time = models.DateTimeField(auto_now_add=True,verbose_name="创建时间")
    owner = models.ForeignKey(User,on_delete=models.CASCADE,verbose_name="作者")
    def __str__(self):
        return self.name
    class Meta:
        verbose_name = verbose_name_plural = '标签'


class Post(models.Model):
    STATUS_NORMAL = 1
    STATUS_DELETE = 0
    STATUS_DRAFT = 2
    STATUS_ITEMS = (
        (STATUS_NORMAL,"正常"),
        (STATUS_DELETE, "删除"),
        (STATUS_DRAFT,"草稿"),
    )
    title = models.CharField(max_length=255, verbose_name = "标题")
    description = models.CharField(max_length=1024, blank = True, verbose_name = "摘要")
    content = models.TextField(verbose_name="正文",help_text="正文必须为MARKDOWN格式")
    content_html = models.TextField(verbose_name="正文HTML",blank = True, editable=False)
    status = models.IntegerField(choices=STATUS_ITEMS, verbose_name="状态", default=STATUS_NORMAL)
    created_time = models.DateTimeField(auto_now_add=True,verbose_name="创建时间")
    owner = models.ForeignKey(User,on_delete=models.CASCADE,verbose_name="作者")
    tags = models.ManyToManyField(Tag,verbose_name="标签")
    category = models.ForeignKey(Category,on_delete=models.CASCADE, verbose_name="分类")
    pv = models.PositiveIntegerField(default=1)
    uv = models.PositiveIntegerField(default=1)

    @classmethod
    def hot_post(cls,with_related = True):
        if with_related:
            return Post.objects.select_related('owner', 'category').filter(status=Post.STATUS_NORMAL).order_by('-pv')
        else:
            return Post.objects.filter(status=Post.STATUS_NORMAL).order_by('-pv')


    def __str__(self):
        return self.title

    @staticmethod
    def get_by_category(category_id):
        try:
            category = Category.objects.get(id = category_id)
        except category.DoesNotExist:
            post_list = []
            category = None
        else:
            post_list = category.post_set.filter(status=Post.STATUS_NORMAL).select_related('owner', 'category')
        return post_list, category

    @staticmethod
    def get_by_tag(tag_id):
        try:
            tag = Tag.objects.get(id = tag_id)
        except tag.DoesNotExist:
            tag = None
            post_list = []
        else:
            post_list = tag.post_set.filter(status=Post.STATUS_NORMAL).select_related('owner','tag')

        return post_list, tag

    @classmethod
    def latest_posts(cls,with_related = True):
        queryset = cls.objects.filter(status = cls.STATUS_NORMAL)
        if with_related:
            queryset = queryset.select_related('owner', 'category')
        return queryset

    def save(self,*args, **kwargs):
        self.content_html = mistune.markdown(self.content)
        super().save(*args, **kwargs)

    @cached_property
    def tagss(self):
        return ','.join(self.tags.values_list('name',flat=True))

    class Meta:
        verbose_name = verbose_name_plural = '文章'
        ordering = ['id']

    class Media:
        css = {
            'all': ("https://cdn.jsdelivr.net/npm/bootstrap@4.6.1/dist/css/bootstrap.min.css",),
        }
        js = ("https://cdn.bootcss.com/bootstrap/4.0.0-beta.2/js/bootstrap.bundle.js")




from dal import autocomplete
from blog.models import  Category,Tag
class CategoryAutocomplete(autocomplete.Select2QuerySetView):
    def get_queryset(self):
        if not self.request.user.is_authenticated:
            return Category.objects.none()
        qs = Category.objects.filter(owner = self.request.user)

        if self.q:
            qs = qs.filter(name__istartswith=self.q)
        return qs

class TagAutocomplete(autocomplete.Select2QuerySetView):
    def get_queryset(self):
        if not self.request.user.is_authenticated:
            return Tag.objects.none()
        qs = Tag.objects.filter(owner = self.request.user)

        if self.q:
            qs = qs.filter(name__istartswith=self.q)

        return qs
